@@ -1,11 +1,11 @@
 ;;; ivy-hydra.el --- Additional key bindings for Ivy  -*- lexical-binding: t -*-
 
-;; Copyright (C) 2015-2019  Free Software Foundation, Inc.
+;; Copyright (C) 2015-2021 Free Software Foundation, Inc.
 
 ;; Author: Oleh Krehel <ohwoeowho@gmail.com>
 ;; URL: https://github.com/abo-abo/swiper
-;; Version: 0.13.0
-;; Package-Requires: ((emacs "24.5") (ivy "0.13.0") (hydra "0.15.0"))
+;; Version: 0.13.4
+;; Package-Requires: ((emacs "24.5") (ivy "0.13.4") (hydra "0.15.0"))
 ;; Keywords: convenience
 
 ;; This file is part of GNU Emacs.
@@ -41,8 +41,21 @@
         (cdr cell)
       "other")))
 
-(defhydra hydra-ivy (:hint nil
-                     :color pink)
+(defun ivy-minibuffer-grow ()
+  "Grow the minibuffer window by 1 line."
+  (interactive)
+  (setq-local max-mini-window-height
+              (cl-incf ivy-height)))
+
+(defun ivy-minibuffer-shrink ()
+  "Shrink the minibuffer window by 1 line."
+  (interactive)
+  (when (> ivy-height 2)
+    (setq-local max-mini-window-height
+                (cl-decf ivy-height))
+    (window-resize nil -1)))
+
+(defhydra hydra-ivy (:hint nil :color pink)
   "
 ^ ^ ^ ^ ^ ^ | ^Call^      ^ ^  | ^Cancel^ | ^Options^ | Action _w_/_s_/_a_: %-14s(ivy-action-name)
 ^-^-^-^-^-^-+-^-^---------^-^--+-^-^------+-^-^-------+-^^^^^^^^^^^^^^^^^^^^^^^^^^^^^---------------------------
@@ -62,7 +75,7 @@ _h_ ^+^ _l_ | _d_one      ^ ^  | _o_ops   | _M_: matcher %-5s(ivy--matcher-desc)
   ("t" ivy-toggle-marks)
   ;; actions
   ("o" keyboard-escape-quit :exit t)
-  ("r" ivy-dispatching-done-hydra :exit t)
+  ("r" ivy-dispatching-done :exit t)
   ("C-g" keyboard-escape-quit :exit t)
   ("i" nil)
   ("C-o" nil)
@@ -84,7 +97,18 @@ _h_ ^+^ _l_ | _d_one      ^ ^  | _o_ops   | _M_: matcher %-5s(ivy--matcher-desc)
   ("U" ivy-occur :exit t)
   ("D" (ivy-exit-with-action
         (lambda (_) (find-function 'hydra-ivy/body)))
-       :exit t))
+   :exit t))
+(dolist (sym '(
+               ;; these cmds have a binding here
+               ivy-next-action ivy-prev-action
+               ivy-unmark-backward ivy-toggle-case-fold
+               ivy-minibuffer-grow ivy-minibuffer-shrink
+               ivy-rotate-preferred-builders ivy-toggle-calling
+               ;; no binding
+               ivy-next-line-or-history ivy-previous-line-or-history
+               ivy-toggle-fuzzy ivy-yank-symbol
+               ivy-occur-next-error))
+  (put sym 'no-counsel-M-x t))
 
 (defvar ivy-dispatching-done-columns 2
   "Number of columns to use if the hint does not fit on one line.")
@@ -94,13 +118,11 @@ _h_ ^+^ _l_ | _d_one      ^ ^  | _o_ops   | _M_: matcher %-5s(ivy--matcher-desc)
 
 (defvar ivy-dispatching-done-hydra-exit-keys '(("M-o" nil "back")
                                                ("C-g" nil))
-  "Keys that can be used to exit `ivy-dispatching-done-hydra'.")
+  "Keys that can be used to exit `ivy-hydra-read-action'.")
 
-(defun ivy-dispatching-done-hydra ()
+(defun ivy-hydra-read-action (actions)
   "Select one of the available actions and call `ivy-done'."
-  (interactive)
-  (let* ((actions (ivy-state-action ivy-last))
-         (extra-actions ivy-dispatching-done-hydra-exit-keys)
+  (let* ((extra-actions ivy-dispatching-done-hydra-exit-keys)
          (doc (concat "action: "
                       (mapconcat
                        (lambda (x) (format "[%s] %s" (nth 0 x) (nth 2 x)))
@@ -120,13 +142,17 @@ _h_ ^+^ _l_ | _d_one      ^ ^  | _o_ops   | _M_: matcher %-5s(ivy--matcher-desc)
            ,@(mapcar (lambda (x)
                        (list (nth 0 x)
                              `(progn
-                                (setcar (ivy-state-action ivy-last) ,(cl-incf i))
-                                (ivy-done))
+                                (let ((prev-idx (car (ivy-state-action ivy-last))))
+                                  (setcar (ivy-state-action ivy-last) ,(cl-incf i))
+                                  ,@(if (eq ivy-exit 'ivy-dispatching-done)
+                                        '((ivy-done))
+                                      '((ivy-call)
+                                        (setcar (ivy-state-action ivy-last) prev-idx)))))
                              (nth 2 x)))
                      (cdr actions))
-           ,@extra-actions))))))
+           ,@extra-actions)))
+      nil)))
 
-(setq ivy-read-action-function (lambda (_) (ivy-dispatching-done-hydra)))
 
 (provide 'ivy-hydra)
 
