@@ -1,6 +1,6 @@
 ;;; ivy-test.el --- tests for ivy -*- lexical-binding: t -*-
 
-;; Copyright (C) 2015-2019  Free Software Foundation, Inc.
+;; Copyright (C) 2015-2021 Free Software Foundation, Inc.
 
 ;; Author: Oleh Krehel
 
@@ -26,29 +26,39 @@
 
 ;;; Code:
 
-(defvar require-features nil)
+(defvar ivy-empty "tests/find-file/empty-dir/")
 
-(defadvice require (before ivy-tests-require-hook (feature &rest _) activate)
-  "Record the requires into `require-features'."
-  (push feature require-features))
+(defvar ivy-features nil
+  "Like `features' but for Ivy testing purposes.")
 
-(require 'ert)
-(require 'colir)
+(defvar ivy-read-hist nil)
+
+(defun ivy-test--record-feature (feature &rest _)
+  "Record FEATURE in `ivy-features'.
+Intended as :after-while advice for `require'."
+  (add-to-list 'ivy-features feature nil #'eq))
+
+(advice-add 'require :after-while #'ivy-test--record-feature)
 
 ;; Useful for #'ivy-read-remap.  It must arrive before (require 'ivy).
 (define-key global-map (kbd "<S-right>") #'end-of-buffer)
 
-(require 'ivy)
+(require 'colir)
 (require 'counsel)
+(require 'ivy)
+
+(require 'ert)
 
 (message "%s" (emacs-version))
 
+(setq ivy-last (make-ivy-state))
+
 (ert-deftest ivy--lazy-load-ffap--ffap-url-p ()
-  (should (not (memq 'ffap require-features)))
+  (should (not (memq 'ffap ivy-features)))
   (should (not (fboundp 'ffap-url-p)))
   (should (string= (ivy-ffap-url-p "https://foo.org")
                    "https://foo.org"))
-  (should (memq 'ffap require-features))
+  (should (memq 'ffap ivy-features))
   (should (fboundp 'ffap-url-p)))
 
 (defvar ivy-expr nil
@@ -80,7 +90,7 @@ Since `execute-kbd-macro' doesn't pick up a let-bound `default-directory'.")
       (unwind-protect
            (progn
              (when dir
-               (setq dir (expand-file-name dir (counsel-locate-git-root))))
+               (setq dir (expand-file-name dir)))
              (setq ivy-eval-dir dir)
              (execute-kbd-macro
               (vconcat (kbd "C-c e")
@@ -93,18 +103,6 @@ Since `execute-kbd-macro' doesn't pick up a let-bound `default-directory'.")
   (setq this-command cmd)
   (apply #'command-execute cmd args))
 
-(defadvice symbol-function (around no-void-function activate)
-  "Suppress void-function errors.
-
-This advice makes `symbol-function' return nil when called on a
-symbol with no function rather than throwing a void-function
-error. On Emacs 24.4 and above, this has no effect, because
-`symbol-function' already does this, but on 24.3 and earlier, it
-will bring the behavior in line with the newer Emacsen."
-  (condition-case nil
-      ad-do-it
-    (void-function nil)))
-
 (ert-deftest ivy-partial-1 ()
   (should (equal
            (ivy-with '(ivy-read "test: " '("case" "Case"))
@@ -179,7 +177,29 @@ will bring the behavior in line with the newer Emacsen."
   (should (equal (ivy-with
                   '(ivy-read "x: " '("one" "two" ("three" . "four")))
                   "th C-m")
-                 "three")))
+                 "three"))
+  (should (equal (ivy-with
+                  '(ivy-read "x: "
+                    (lambda (_input)
+                      '(("one" . 1)
+                        ("two" . 2)))
+                    :dynamic-collection t
+                    :action (lambda (choice)
+                              (message "%s" choice)))
+                  "C-m")
+                 "one")))
+
+(ert-deftest ivy-read-history ()
+  (should (equal (progn
+                   (setq ivy-read-hist '("c" "b" "a"))
+                   (ivy-with '(ivy-read "test: " '("c" "d") :history 'ivy-read-hist) "RET")
+                   ivy-read-hist)
+                 '("c" "b" "a")))
+  (should (equal (progn
+                   (setq ivy-read-hist '("cdef" "b" "a"))
+                   (ivy-with '(ivy-read "test: " '("cdef" "g") :history 'ivy-read-hist) "cd RET")
+                   ivy-read-hist)
+                 '("cd" "cdef" "b" "a"))))
 
 (ert-deftest ivy-read-sort-alist ()
   (should (equal (ivy-with '(let ((coll '(("b" . "1") ("a" . "2"))))
@@ -189,6 +209,65 @@ will bring the behavior in line with the newer Emacsen."
                            "C-m")
                  '(("b" . "1") ("a" . "2")))))
 
+(ert-deftest ivy-read-alist-multi-cands ()
+  (should
+   (equal
+    (ivy-with '(let (acc)
+                 (ivy-read "test: "
+                           '(("Key 1" . "Data 1") ("Key 2" . "Data 2"))
+                           :action (lambda (x) (push x acc)))
+                 acc)
+              "M-a RET")
+    '(("Key 2" . "Data 2")
+      ("Key 1" . "Data 1"))))
+  (should
+   (equal
+    (ivy-with
+     '(let (res)
+        (ivy-read "test: "
+                  '(("Key 1" . "Data 1") ("Key 2" . "Data 2"))
+                  :action (lambda (x) (push x res))
+                  :multi-action (lambda (xs) (setq res xs)))
+
+        res)
+     "M-a RET")
+    '(("Key 1" . "Data 1")
+      ("Key 2" . "Data 2")))))
+
+(ert-deftest ivy-read-multi-action-1 ()
+  (should (equal (let ((res nil))
+                   (ivy-add-actions 'test-ivy-read-multi-action-1
+                                    (list (list "a" (lambda (x) (push x res)) "action-a")))
+                   (ivy-with
+                    '(ivy-read "test: " '("x" "y")
+                               :action (lambda (x) (message "Default: %s" x))
+                               :multi-action (lambda (xs) (message "Default: %S" xs))
+                               :caller 'test-ivy-read-multi-action-1)
+                    "M-a M-o a")
+                   res)
+                 '("y" "x"))))
+
+(ert-deftest ivy-read-multi-action-2 ()
+  (should (equal (let ((res nil))
+                   (ivy-add-actions 'test-ivy-read-multi-action-2
+                                    (list (list "a"
+                                                (lambda (x) (push x res))
+                                                "action-a"
+                                                (lambda (xs) (push xs res)))))
+                   (ivy-with
+                    '(ivy-read "test: " '("x" "y")
+                               :action (lambda (x) (message "Default: %s" x))
+                               :multi-action (lambda (_xs) (message "default"))
+                               :caller 'test-ivy-read-multi-action-2)
+                    "M-a M-o a")
+                   res)
+                 '(("x" "y")))))
+
+(ert-deftest ivy-read-sort-def ()
+  (should (equal (ivy-with '(ivy-read "Test: " '("1" "2") :def '("a" "b" "c"))
+                           "C-m")
+                 "a")))
+
 (ert-deftest ivy-read-remap ()
   (should (equal
            (ivy-with '(ivy-read "pattern: " '("blue" "yellow" "red"))
@@ -197,12 +276,13 @@ will bring the behavior in line with the newer Emacsen."
 
 (ert-deftest swiper--re-builder ()
   (setq swiper--width 4)
+  (setf (ivy-state-caller ivy-last) 'swiper)
   (should (string= (swiper--re-builder "^")
-                   "."))
+                   "^ "))
   (should (string= (swiper--re-builder "^a")
-                   "^ ?\\(a\\)"))
+                   "^ a"))
   (should (string= (swiper--re-builder "^a b")
-                   "^ \\(a\\).*?\\(b\\)"))
+                   "\\(^ a\\).*?\\(b\\)"))
   (should
    (string-match-p
     "\\`\\\\_<.*\\\\_>\\'"
@@ -230,7 +310,27 @@ will bring the behavior in line with the newer Emacsen."
                    "your  king.")))
   (should (equal (ivy--split "^[^ ]") '("^[^ ]")))
   (should (equal (ivy--split "^[^ ] bar") '("^[^ ]" "bar")))
-  (should (equal (ivy--split "defun [^ ]+") '("defun" "[^ ]+"))))
+  (should (equal (ivy--split "defun [^ ]+") '("defun" "[^ ]+")))
+  (should (equal (ivy--split "[^ ]+ -> .*")
+                 '("[^ ]+" "->" ".*")))
+  (should (equal (ivy--split "[^ \n]+ \\( ->\\)")
+                 '("[^ \n]+" "\\( ->\\)")))
+  (should (equal (ivy--split "abc[^ \n]+\\( ->\\)")
+                 '("abc[^ \n]+" "\\( ->\\)")))
+  (should (equal (ivy--split "abc[^ \n]+\\( -> \\)def")
+                 '("abc[^ \n]+" "\\( -> \\)" "def")))
+  (should (equal (ivy--split "\\(?:interactive\\|swiper\\) \\(?:list\\|symbol\\)")
+                 '("\\(?:interactive\\|swiper\\)" "\\(?:list\\|symbol\\)")))
+  (should (equal (ivy--split "\\([^ \n]+\\)\\( -> \\).*")
+                 '("\\([^ \n]+\\)"
+                   "\\( -> \\)"
+                   ".*")))
+  (should (equal (ivy--split "[^ ]\\( -> \\).*")
+                 '("[^ ]" "\\( -> \\)" ".*")))
+  (should (equal (ivy--split "[ab][cd]") '("[ab][cd]")))
+  (should (equal (ivy--split "[a b][c d]") '("[a b][c d]")))
+  (should (equal (ivy--split "[ab] [cd]") '("[ab]" "[cd]")))
+  (should (equal (ivy--split "[a b] [c d]") '("[a b]" "[c d]"))))
 
 (ert-deftest ivy--regex ()
   (should (equal (ivy--regex
@@ -246,7 +346,12 @@ will bring the behavior in line with the newer Emacsen."
                   ".org")
                  "\\.org"))
   (should (equal (ivy--regex "foo\\") "foo"))
-  (should (equal (ivy--regex "foo\\|") "foo")))
+  (should (equal (ivy--regex "foo\\|") "foo"))
+  (should (equal (ivy--regex "[^ \n]+\\( -> \\).*")
+                 "\\([^ \n]+\\)\\( -> \\).*?\\(.*\\)"))
+  (should (equal (ivy--regex "\\([^ \\n]+\\)\\( -> \\).*")
+                 "\\([^ \\n]+\\)\\( -> \\).*?\\(.*\\)"))
+  (should (equal (ivy--regex "\\\\") "\\\\")))
 
 (ert-deftest ivy--split-negation ()
   (should (equal (ivy--split-negation "") ()))
@@ -293,13 +398,9 @@ will bring the behavior in line with the newer Emacsen."
   (should (equal (ivy--regex-plus "add path\\!") "\\(add\\).*?\\(path!\\)")))
 
 (ert-deftest ivy-partial-2 ()
-  (when (fboundp 'read--expression)
-    (should
-     (equal
-      (ivy-with '(read--expression "Eval: "
-                  "'s-c-t-st")
-                "<tab> C-m")
-      '(quote shell-command-to-string)))))
+  (should (equal (ivy-with '(read--expression "Eval: " "'s-c-t-st")
+                           "<tab> C-m")
+                 '(quote shell-command-to-string))))
 
 (ert-deftest ivy--regex-fuzzy ()
   (should (string= (ivy--regex-fuzzy "tmux")
@@ -374,7 +475,6 @@ will bring the behavior in line with the newer Emacsen."
                      90 96 (face ivy-current-match read-only nil)))))
 
 (ert-deftest ivy--filter ()
-  (setq ivy-last (make-ivy-state))
   (should (equal (ivy--filter "the" '("foo" "the" "The"))
                  '("the" "The")))
   (should (equal (ivy--filter "The" '("foo" "the" "The"))
@@ -416,6 +516,18 @@ will bring the behavior in line with the newer Emacsen."
      (insert ,text)
      ,@body))
 
+(ert-deftest ivy-backward-kill-word ()
+  (should (string= (ivy-with
+                    '(ivy-read "test: " nil
+                      :initial-input "one two three")
+                    "M-DEL M-DEL C-M-j")
+                   "one "))
+  (should (string= (ivy-with
+                    '(ivy-read "test: " nil
+                      :initial-input "one two three")
+                    "M-DEL M-DEL M-DEL C-y C-M-j")
+                   "one two three")))
+
 (ert-deftest counsel-url-expand ()
   "Test ffap expansion using `counsel-url-expansions-alist'."
   ;; no expansions defined
@@ -424,8 +536,8 @@ will bring the behavior in line with the newer Emacsen."
   (let ((counsel-url-expansions-alist
          '(("^foo$" . "https://foo.com/%s")
            ("^issue\\([0-9]+\\)" . (lambda (word)
-                                     (concat "https://foo.com/issues/"
-                                             (match-string 1 word)))))))
+                                 (concat "https://foo.com/issues/"
+                                         (match-string 1 word)))))))
     ;; no match
     (should (equal (ivy--string-buffer
                     "foobar"
@@ -774,6 +886,20 @@ will bring the behavior in line with the newer Emacsen."
             "C-M-i")
            "(nconc")))
 
+(ert-deftest ivy-completing-read ()
+  (should (equal (ivy-with '(ivy-completing-read
+                             "Test: " '(("1" . "a") ("2" . "b")))
+                           "RET")
+                 "1"))
+  (should (equal (progn
+                   (setq ivy-read-hist '("foo"))
+                   (ivy-with
+                    '(completing-read "test: " '("foo" "bar" "baz") nil t nil
+                      'ivy-read-hist)
+                    "fo RET")
+                   ivy-read-hist)
+                 '("foo"))))
+
 (ert-deftest ivy-completing-read-def-handling ()
   ;; DEF in COLLECTION
   (should
@@ -951,6 +1077,12 @@ will bring the behavior in line with the newer Emacsen."
 
 (ert-deftest ivy-read-directory-name ()
   (ivy-mode 1)
+  (unless (file-exists-p ivy-empty)
+    (make-directory ivy-empty))
+  (should (equal (expand-file-name ivy-empty)
+                 (ivy-with
+                  '(read-directory-name "cd: " ivy-empty nil t)
+                  "RET")))
   (should
    (equal (expand-file-name "/tmp/")
           (ivy-with
@@ -975,6 +1107,26 @@ will bring the behavior in line with the newer Emacsen."
            "DEL C-M-j"
            :dir "/tmp"))))
 
+(ert-deftest ivy-read-file-name-initial-input ()
+  (let ((fname (expand-file-name "ivy.el")))
+    (should (string=
+             fname
+             (ivy-with
+              `(ivy-read "Find file: " 'read-file-name-internal
+                         :predicate 'file-exists-p
+                         :require-match 'confirm-after-completion
+                         :initial-input ,fname
+                         :preselect ,fname
+                         :def ,fname
+                         :history 'file-name-history
+                         :keymap nil
+                         :sort t
+                         :dynamic-collection nil
+                         :caller 'read-file-name-internal
+                         :action (lambda (x) x))
+              "RET"))))
+  (should (string= (ivy-state-initial-input ivy-last) "ivy.el")))
+
 (ert-deftest ivy-counsel-read-directory-name ()
   (should
    (equal (expand-file-name "/tmp/")
@@ -1089,14 +1241,14 @@ a buffer visiting a file."
   ;; negative lookahead: lines with "ivy", without "-"
   (should
    (string=
-    (let ((counsel--regex-look-around t)
-          (ivy--regex-function 'ivy--regex-plus))
+    (cl-letf ((counsel--regex-look-around t)
+              ((ivy-state-re-builder ivy-last) #'ivy--regex-plus))
       (counsel--grep-regex "ivy ! -"))
     "^(?=.*ivy)(?!.*-)"))
   (should
    (string=
-    (let ((counsel--regex-look-around t)
-          (ivy--regex-function 'ivy--regex-fuzzy))
+    (cl-letf ((counsel--regex-look-around t)
+              ((ivy-state-re-builder ivy-last) #'ivy--regex-fuzzy))
       (counsel--grep-regex "ivy"))
     "(i)[^v\n]*(v)[^y\n]*(y)")))
 
@@ -1136,6 +1288,16 @@ a buffer visiting a file."
            (and (buffer-name temp-buffer)
                 (kill-buffer temp-buffer)))))))
 
+(ert-deftest swiper-query-replace ()
+  (dolist (re-builder '(regexp-quote ivy--regex ivy--regex-plus ivy--regex-fuzzy ivy--regex-ignore-order))
+    (dolist (swiper-cmd '(swiper swiper-isearch))
+      (let ((ivy-re-builders-alist `((t . ,re-builder))))
+        (should (equal (ivy-with-text
+                        "|foo bar"
+                        (global-set-key (kbd "C-s") swiper-cmd)
+                        ("C-s" "foo" "M-q" "asdf" "C-j" "y"))
+                       "asdf| bar"))))))
+
 (ert-deftest swiper-thing-at-point ()
   (should
    (string=
@@ -1289,11 +1451,9 @@ a buffer visiting a file."
     "Foo\nfoo|\nFOO\n")))
 
 (ert-deftest ivy-swiper-wgrep ()
-  :expected-result (if (and (= emacs-major-version 24)
-                            (<= emacs-minor-version 3))
-                       ;; `wgrep' requires at least 24.5
-                       :failed
-                     :passed)
+  ;; `wgrep' requires Emacs 25 or later.
+  (skip-unless (and (>= emacs-major-version 25)
+                    (require 'wgrep nil t)))
   (dolist (search-cmd '(swiper swiper-isearch))
     (should
      (string=
@@ -1317,18 +1477,19 @@ a buffer visiting a file."
     (insert
      "line0\nline1\nline line\nline line\nline5")
     (let* ((input "li")
-           (cands (swiper--isearch-function input))
+           (cands (progn
+                    (ivy-set-text input)
+                    (swiper--isearch-function input)))
            (len (length cands)))
       (should (equal cands '(3 9 15 20 25 30 35)))
       (dotimes (index len)
-        (should (string= (substring-no-properties
-                          (swiper--isearch-format
-                           index len
-                           cands
-                           input
-                           (nth index cands)
-                           (current-buffer)))
-                         "line0\nline1\nline line\nline line\nline5"))))))
+        (should (equal (swiper--isearch-format
+                        index len
+                        cands
+                        input
+                        (nth index cands)
+                        (current-buffer))
+                       '("line0" "line1" "line line" "line line" "line5")))))))
 
 (ert-deftest ivy-use-selectable-prompt ()
   (let ((ivy-use-selectable-prompt t)
@@ -1394,12 +1555,43 @@ a buffer visiting a file."
                         :dir "tests/find-file/directories-with-spaces/"))
              "tests/find-file/directories-with-spaces/bar baz ii/file2"))))
 
+(ert-deftest counsel-find-file-single-match-directories ()
+  (dolist (ivy-re-builders-alist '(((t . ivy--regex-plus))
+                                   ((t . ivy--regex-ignore-order))))
+    (should (string= (let ((ivy-extra-directories nil))
+                       (file-relative-name
+                        (ivy-with '(counsel-find-file) "a TAB TAB TAB"
+                                  :dir "tests/find-file/single-match-directories/")))
+                     "tests/find-file/single-match-directories/a/file_in_a.txt"))
+    (should (string= (let ((ivy-extra-directories nil))
+                       (file-relative-name
+                        (ivy-with '(counsel-find-file) "b TAB TAB TAB"
+                                  :dir "tests/find-file/single-match-directories/")))
+                     "tests/find-file/single-match-directories/ba/file_in_ba.txt"))))
+
+(ert-deftest counsel--split-string-with-eol-cr ()
+  (should
+   (equal (counsel--split-string "one\rtwo")
+          '("one" "two"))))
+
+(ert-deftest counsel--split-string-with-eol-lf ()
+  (should
+     (equal (counsel--split-string "one\ntwo")
+            '("one" "two"))))
+
+(ert-deftest counsel--split-string-with-eol-crlf ()
+  (should
+     (equal (counsel--split-string "one\r\ntwo")
+            '("one" "two"))))
+
 (ert-deftest ivy-avy ()
-  (when (require 'avy nil t)
-    (let ((enable-recursive-minibuffers t)
-          (read-numbers '(ivy-read "test: " (mapcar #'number-to-string (number-sequence 1 100)))))
+  (skip-unless (require 'avy nil t))
+  (require 'ivy-avy)
+  (let ((enable-recursive-minibuffers t)
+        (read-numbers '(ivy-read "test: " (mapcar #'number-to-string
+                                                  (number-sequence 1 100)))))
       (should (string= (ivy-with read-numbers "C-' a") "1"))
-      (should (string= (ivy-with read-numbers "C-v C-' d") "7")))))
+      (should (string= (ivy-with read-numbers "C-v C-' d") "7"))))
 
 (ert-deftest ivy--yank-handle-case-fold ()
   (should (string=
@@ -1417,6 +1609,85 @@ a buffer visiting a file."
                      (ivy--handle-directory "/sudo::"))
                    "/sudo::/tmp/")))
 
+(ert-deftest ivy--handle-full-path-yank-on-remote ()
+  (should
+   (string=
+    (let ((ivy--directory "/ssh:dev:/bin/"))
+      (ivy--expand-file-name "/etc/hosts"))
+    "/ssh:dev:/etc/hosts")))
+
+(ert-deftest ivy-inhibit-action ()
+  (should (equal (ivy-with
+                  '(let ((ivy-inhibit-action #'identity))
+                    (ivy-read "pattern: " '(("a" . 1) ("b" . 2))))
+                  "C-m")
+                 '("a" . 1)))
+  (should (equal (ivy-with
+                  '(let ((ivy-inhibit-action #'cdr))
+                    (ivy-read "pattern: " '(("a" . 1) ("b" . 2))))
+                  "C-n C-m")
+                 2)))
+
+(ert-deftest ivy-empty-directory-open ()
+  (unless (file-exists-p ivy-empty)
+    (make-directory ivy-empty))
+  (should (string= (file-relative-name
+                    (ivy-with '(counsel-find-file)
+                              "RET"
+                              :dir ivy-empty))
+                   ivy-empty)))
+
+(ert-deftest counsel--split-command-args ()
+  (should (equal
+           (counsel--split-command-args "require -- -g*.el")
+           '("-g*.el" . "require")))
+  (should (equal
+           (counsel--split-command-args "-g*.el -- require")
+           '("-g*.el" . "require")))
+  (should (equal
+           (counsel--split-command-args "counsel--format")
+           '("" . "counsel--format"))))
+
+(ert-deftest ivy--preselect-index ()
+  "Test `ivy--preselect-index' behavior."
+  (should (eql (ivy--preselect-index nil ()) 0))
+  (should (eql (ivy--preselect-index nil '(nil)) 0))
+  (should (eql (ivy--preselect-index nil '(t)) 0))
+  (should (eql (ivy--preselect-index nil '(t nil)) 1))
+  (should (eql (ivy--preselect-index 0 ()) 0))
+  (should (eql (ivy--preselect-index 0 '(0)) 0))
+  (should (eql (ivy--preselect-index 0 '(1)) 0))
+  (should (eql (ivy--preselect-index 0 '(1 0)) 1))
+  (should (eql (ivy--preselect-index 0 '(a)) 0))
+  (should (eql (ivy--preselect-index 1 '(a)) 1))
+  (should (eql (ivy--preselect-index "" ()) 0))
+  (should (eql (ivy--preselect-index "" '("")) 0))
+  (should (eql (ivy--preselect-index "" '("a")) 0))
+  (should (eql (ivy--preselect-index "a+" '("a")) 0))
+  (should (eql (ivy--preselect-index "a+" '("b" "a")) 1)))
+
+(ert-deftest ivy-multi-resume ()
+  (let ((ivy-last ivy-last)
+        ivy-text ivy--all-candidates ivy--sessions)
+    (ivy-with '(ivy-read "A: " '(a123 b456 c789) :caller 'test-a :action #'ignore)
+              "b4 RET")
+    (ivy-with '(ivy-read "A: " '(d123 e456 f789) :caller 'test-b)
+              "d1 RET")
+    (ivy-with '(ivy-read "A: " '(g123 h456 k789) :action #'ignore
+                         :extra-props '(:session test-c))
+              "k7 RET")
+    (should (equal ivy-text "k7"))
+    (should (equal (mapcar #'car ivy--sessions) '(test-c test-a)))
+    (should (equal (ivy-with '(let ((current-prefix-arg '(4)))
+                               (ivy-resume))
+                             "test-a RET RET")
+                   "b456"))
+    (should (equal ivy-text "b4"))
+
+    (should (equal (ivy-with '(ivy-resume 'test-c) "RET")
+                   "k789"))
+    (should (equal ivy-text "k7"))))
+
 (defun ivy-test-run-tests ()
   (let ((test-sets
          '(
